package jp.gr.java_conf.coskx.gles2sample14;

import android.opengl.GLES20;

import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

/**
 * Created by tommy on 2016/10/13.
 * 部分円錐形を描画するクラス
 * 特別な場合として，次のものが描画できる
 * （１）円柱　ただし，二底面は描画されない
 * （２）円錐台　ただし，二底面は描画されない
 * （３）円錐　ただし，底面は描画されない
 * （４）平面リング　アニュラス（annulus）
 * （５）円板
 * （１）-（３）の下底面はzx平面にある。（４）（５）はzx平面内でy+が表になるように描かれる
 * （１）-（３）の下底面の半径は１である。（４）（５）の外径は１である。
 * （１）-（３）の高さはheightで与える。（４）（５）を描くときheght=0とする。
 * （１）（２）の上底面の半径はradius2で与える。（４）の内径はradius2で与える
 * （３）（５）を描くときはradius2=0とする
 * 部分を描くときは0からmaxlongitude[deg]までを描く。
 * maxlongitudeは+z方向を0としてy軸回りに測る
 *
 * インスタンス生成例
 * ConeEX mConeEX = new ConeEX(1f,1f,360f,64); //円柱　底面はない
 * ConeEX mConeEX = new ConeEX(1f,.5f,360f,64); //円錐台　底面はない
 * ConeEX mConeEX = new ConeEX(1f,0f,360f,64); //円錐　底面はない
 * ConeEX mConeEX = new ConeEX(0f,.5f,360f,64); //平面リング
 * ConeEX mConeEX = new ConeEX(0f,0f,360f,64); //円板
 * ConeEX mConeEX = new ConeEX(0f,0f,180f,64); //半円板
 */

public class ConeEX {
    ConeEX(){}
    ConeEX(float height, float radius2, float maxlongitude, int nSlices) {
        makeCone(height, radius2, maxlongitude, nSlices);
    }

    //bufferの定義
    protected FloatBuffer vertexBuffer;
    protected ShortBuffer indexBuffer;
    protected FloatBuffer normalBuffer;

    protected int nIndexs;
    protected int nPoints;

    protected void makeCone(float height, float radius2, float maxlongitude, int nSlices) {
        int nSlices1=nSlices+1;
        nPoints = nSlices1*2;
        double dlongitude=3.141592653589793/180.*maxlongitude/nSlices;
        double longitude;
        int sizeArray=nPoints*3;
        int p;

        //頂点3Dベクトル
        float[] vertexs= new float[sizeArray];
        p=0;
        for (int i=0; i<nSlices1; i++) {
            longitude = i*dlongitude;
            vertexs[p++]=(float)Math.sin(longitude);            //x
            vertexs[p++]=0f;                                     //y
            vertexs[p++]=(float)Math.cos(longitude);            //z
            vertexs[p++]=radius2*(float)Math.sin(longitude);    //x
            vertexs[p++]=height;                                 //y
            vertexs[p++]=radius2*(float)Math.cos(longitude);    //z
        }
        /************************************
         nSlices = 5 のとき
         1 3 5 7 9 11
         0 2 4 6 8 10
         *************************************/

        //頂点の3D法線ベクトル
        float[] normals= new float[sizeArray];
        p=0;
        double theta=Math.atan2((1f-radius2),height);
        float z0=(float)Math.cos(theta);
        float y0=(float)Math.sin(theta);
        for (int i=0;i<nSlices1;i++) {
            longitude = i*dlongitude;
            normals[p++]=z0*(float)Math.sin(longitude);
            normals[p++]=y0;
            normals[p++]=z0*(float)Math.cos(longitude);
            normals[p]=normals[p-3]; p++;
            normals[p]=normals[p-3]; p++;
            normals[p]=normals[p-3]; p++;
        }

        //頂点座標番号列
        nIndexs=4*nSlices;
        short [] indexs= new short[nIndexs];
        p=0;
        for (int i=0;i<nSlices;i++) {
            for (int j=0;j<2;j++) {
                indexs[p++]=(short)(i*2+j);
                indexs[p++]=(short)((i+1)*2+j);
            }
        }
        vertexBuffer=BufferUtil.makeFloatBuffer(vertexs);
        indexBuffer=BufferUtil.makeShortBuffer(indexs);
        normalBuffer=BufferUtil.makeFloatBuffer(normals);
    }

    public void draw(float r,float g,float b,float a, float shininess){
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        if (GLES.checkLiting()) {
            //頂点での法線ベクトル
            GLES20.glVertexAttribPointer(GLES.normalHandle, 3,
                    GLES20.GL_FLOAT, false, 0, normalBuffer);

            //周辺光反射
            GLES20.glUniform4f(GLES.materialAmbientHandle, r, g, b, a);

            //拡散反射
            GLES20.glUniform4f(GLES.materialDiffuseHandle, r, g, b, a);

            //鏡面反射
            GLES20.glUniform4f(GLES.materialSpecularHandle, 1f, 1f, 1f, a);
            GLES20.glUniform1f(GLES.materialShininessHandle, shininess);

        } else {
            //shadingを使わない時に使う単色の設定 (r, g, b,a)
            GLES20.glUniform4f(GLES.objectColorHandle, r, g, b, a);
        }

        //
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_TRIANGLE_STRIP,
                nIndexs, GLES20.GL_UNSIGNED_SHORT, indexBuffer);

    }
}
