package jp.gr.java_conf.coskx.gles2sample14;

import android.opengl.GLES20;
import android.opengl.Matrix;

import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

/**
 * Created by tommy on 2016/10/15.
 * トーラス（ドーナツ形）および部分トーラスを描くクラスである
 * 点(x=0,y=0,z=1)を中心にzx平面内に半径radius2の円Cを描き、
 * その円をy軸中心に経度最大値maxlongitude[deg]まで回転させた図形を描く
 * maxlongitude[deg]=360とすると、トーラスが完成する
 * 円Cの一部を使って、緯度最小値minlatitude[deg]から緯度最大値maxlatitude[deg]までで円弧を作り、
 * y軸中心に経度最大値maxlongitude[deg]まで回転させた図形を描くこともできる。
 * minlatitude < maxlatitude でなければならないが、これらの値の範囲は0-360である必要はない。
 *
 * インスタンス生成例
 * TorusEX mTorusEX = new TorusEX(.5f, 360f,0f,360f,72,36); //トーラスを定義
 * TorusEX mTorusEX = new TorusEX(.5f, 180f,0f,360f,72,36); //半周トーラスを定義
 * TorusEX mTorusEX = new TorusEX(.5f, 360f,0f,180f,72,36); //上半分トーラスを定義
 */

public class TorusEX {
    TorusEX() {
    }

    TorusEX(float radius2, float maxlongitude, float minlatitude, float maxlatitude, int nSlices, int nStacks) {
        makeTorus(radius2, maxlongitude, minlatitude, maxlatitude, nSlices, nStacks);
    }

    //bufferの定義
    protected FloatBuffer vertexBuffer;
    protected ShortBuffer indexBuffer;
    protected FloatBuffer normalBuffer;

    protected int nIndexs;
    protected int nPoints;

    protected void makeTorus(float radius2, float maxlongitude, float minlatitude, float maxlatitude, int nSlices, int nStacks) {
        int nSlices1=nSlices+1;
        int nStacks1=nStacks+1;
        nPoints = nSlices1*nStacks1;
        float[] mMatrix = new float[16];
        int sizeArray=nPoints*3;
        double latitude;
        float longitude;
        double dlatitude=3.141592653589793/180.*(maxlatitude-minlatitude)/nStacks;
        double minlatitudeR=3.141592653589793/180.*minlatitude;
        int p;
        p=0;
        float[] tmpvertexs= new float[nStacks1*4]; //原点中心　ZX平面内に半径radius2の円を作る
        for (int i=0; i<nStacks1; i++) {
            latitude = minlatitudeR + i * dlatitude;
            tmpvertexs[p++]=0f;
            tmpvertexs[p++]=radius2*(float)Math.sin(latitude);
            tmpvertexs[p++]=radius2*(float)Math.cos(latitude);
            tmpvertexs[p++]=1f;
        }

        //頂点3Dベクトル
        float[] vertexs= new float[sizeArray];
        float[] vers= new float[4];
        float dlongitude=maxlongitude/nSlices;
        for (int i=0; i<nSlices1; i++) {
            longitude = i * dlongitude;
            Matrix.setIdentityM(mMatrix,0);
            Matrix.rotateM(mMatrix,0,longitude,0f,1f,0f);
            Matrix.translateM(mMatrix,0,0f,0f,1f);
            for (int j=0; j<nStacks1; j++) {
                Matrix.multiplyMV(vers,0,mMatrix,0,tmpvertexs,j*4);
                System.arraycopy(vers,0,vertexs,(i*nStacks1+j)*3,3);
            }
        }
        /************************************
         nStacks = 4, nSlices = 5 のとき
         4 9 14 19 24 29
         3 8 13 18 23 28
         2 7 12 17 22 27
         1 6 11 16 21 26
         0 5 10 15 20 25
         *************************************/

        //頂点の3D法線ベクトル
        float[] normals= new float[sizeArray];
        for (int i=0; i<nSlices1; i++) {
            longitude = i * dlongitude;
            Matrix.setIdentityM(mMatrix,0);
            Matrix.rotateM(mMatrix,0,longitude,0f,1f,0f);
            for (int j=0; j<nStacks1; j++) {
                Matrix.multiplyMV(vers,0,mMatrix,0,tmpvertexs,j*4);
                System.arraycopy(vers,0,normals,(i*nStacks1+j)*3,3);
            }
        }

        //頂点座標番号列
        nIndexs=nStacks1*2*nSlices;
        short [] indexs= new short[nIndexs];
        p=0;
        for (int i=0;i<nSlices;i++) {
            for (int j=0;j<nStacks1;j++) {
                indexs[p++]=(short)(i*nStacks1+j);
                indexs[p++]=(short)((i+1)*nStacks1+j);
            }
        }

        vertexBuffer=BufferUtil.makeFloatBuffer(vertexs);
        indexBuffer=BufferUtil.makeShortBuffer(indexs);
        normalBuffer=BufferUtil.makeFloatBuffer(normals);
    }

    public void draw(float r,float g,float b,float a, float shininess){
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        if (GLES.checkLiting()) {
            //頂点での法線ベクトル
            GLES20.glVertexAttribPointer(GLES.normalHandle, 3,
                    GLES20.GL_FLOAT, false, 0, normalBuffer);

            //周辺光反射
            GLES20.glUniform4f(GLES.materialAmbientHandle, r, g, b, a);

            //拡散反射
            GLES20.glUniform4f(GLES.materialDiffuseHandle, r, g, b, a);

            //鏡面反射
            GLES20.glUniform4f(GLES.materialSpecularHandle, 1f, 1f, 1f, a);
            GLES20.glUniform1f(GLES.materialShininessHandle, shininess);

        } else {
            //shadingを使わない時に使う単色の設定 (r, g, b,a)
            GLES20.glUniform4f(GLES.objectColorHandle, r, g, b, a);
        }

        //
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_TRIANGLE_STRIP,
                nIndexs, GLES20.GL_UNSIGNED_SHORT, indexBuffer);

    }
}
